/********************************************************************************
  Copyright: 2016-2019 ROS小课堂 www.corvin.cn
  Author: corvin
  Description:
       使用dfrobot的arduino UNO开发板，再接扩展板。所有的舵机都接在扩展板上,这样方便接线.
    当然其他公司的UNO板也行，在控制转动前需要先使能指定pwm引脚上的舵机，否则发送转动舵机的命
    令是不执行的.具体操作命令格式如下：
    (0).使能pwm 2号引脚上的舵机,该引脚舵机索引值为0,第2个参数表明使能（非0）或禁用（0）：
      e 0 1
    (1).控制舵机旋转，舵机索引分别为为0，1。后面接着要转动到的角度，最后的数字为移动的延时参数:
      w 0 90 20:该串口命令中w表示写入，0表示0号舵机，90表示旋转到90度,每次移动延时20ms
      w 1 30 50:表示1号舵机移动到30度,每个移动周期延时50ms.
    (2).读取指定舵机ID的当前旋转到的角度：
      r 0:串口输入该命令，则返回90,表示0号舵机当前在90度位置.
      r 1:输入该命令后，返回30,表示1号舵机当前在30度位置。
    (3).获取所有舵机的使能状态命令s:
      s:0 0 0 1 0 0：返回的一串数字表明第几个舵机使能。
    (4).获取当前所有舵机的角度p:
      p:0 0 0 90 0 0:第4个舵机当前在90度位置。
  History:
    20181225: initial this code.
********************************************************************************/
#define  FIRMWARE_VERSION   1
#define  CONNECT_BAUDRATE   57600

#include <Servo.h>
#include "serialData.h"
#include "commands.h"
#include "servos.h"

/* Run a command.  Commands are defined in commands.h */
void runCommand(void)
{
  serialObj.arg1 = atoi(serialObj.argv1);
  serialObj.arg2 = atoi(serialObj.argv2);
  serialObj.arg3 = atoi(serialObj.argv3);

  switch (serialObj.cmd_chr)
  {
    case GET_CONNECT_BAUDRATE:  // 'b':get connect baud rate.
      Serial.println(CONNECT_BAUDRATE);
      break;

    case SET_ONE_SERVO_ENABLE:  // 'e':enable one servo.
      myServos[serialObj.arg1].setEnable(serialObj.arg2);
      Serial.println("OK");
      break;

    case SET_ONE_SERVO_POS:   // 'w':set one servo target position.
      myServos[serialObj.arg1].setTargetPos(serialObj.arg2, serialObj.arg3);
      Serial.println("OK");
      break;

    case GET_ONE_SERVO_POS:   // 'r':get one servo current position.
      Serial.println(myServos[serialObj.arg1].getServoObj().read());
      break;

    case GET_ALL_SERVOS_POS: // 'p':get all servos current position.
      for (byte i = 0; i < N_SERVOS; i++)
      {
        Serial.print(myServos[i].getCurrentPos());
        Serial.print(' ');
      }
      Serial.println("");
      break;

    case GET_ALL_SERVOS_ENABLE: // 's':get all servo enable status.
      for (byte i = 0; i < N_SERVOS; i++)
      {
        Serial.print(myServos[i].isEnabled());
        Serial.print(' ');
      }
      Serial.println("");
      break;

    case GET_FIRMWARE_VERSION:  // 'v':get current firmware version.
      Serial.println(FIRMWARE_VERSION);
      break;

    default:
      Serial.println("Invalid Command");
      break;
  }
}

/* Setup function--runs once at startup. */
void setup()
{
  Serial.begin(CONNECT_BAUDRATE);
  serialObj.resetCmdParam();

  /* when power on init all servos position */
  for (byte i = 0; i < N_SERVOS; i++)
  {
    myServos[i].initServo(myServoPins[i], servoInitPosition[i], 0);
  }
}

/* Read and parse input from the serial port and run any valid commands. */
void loop()
{
  while (Serial.available() > 0)
  {
    char tmp_chr = Serial.read(); //Read the next character

    if (tmp_chr == 13) // Terminate a command with a CR
    {
      runCommand();
      serialObj.resetCmdParam();
    }
    else if (tmp_chr == ' ') // Use spaces to delimit parts of the command
    {
      if (serialObj.argCnt == 0)
      {
        serialObj.argCnt++;
      }
      else if (serialObj.argCnt == 1)
      {
        serialObj.argCnt++;
        serialObj.argIndex = 0;
      }
      else if (serialObj.argCnt == 2)
      {
        serialObj.argCnt++;
        serialObj.argIndex = 0;
      }
    }
    else //parse valid param
    {
      if (serialObj.argCnt == 0) // The first arg is the char command
      {
        serialObj.cmd_chr = tmp_chr;
      }
      else if (serialObj.argCnt == 1) // Get char cmd's first param
      {
        serialObj.argv1[serialObj.argIndex] = tmp_chr;
        serialObj.argIndex++;
      }
      else if (serialObj.argCnt == 2)
      {
        serialObj.argv2[serialObj.argIndex] = tmp_chr;
        serialObj.argIndex++;
      }
      else if (serialObj.argCnt == 3)
      {
        serialObj.argv3[serialObj.argIndex] = tmp_chr;
        serialObj.argIndex++;
      }
    }
  }//end while

  // Check everyone servo if isEnabled, when true will move servo. Other don't move servo.
  for (byte i = 0; i < N_SERVOS; i++)
  {
    if (myServos[i].isEnabled())
    {
      myServos[i].moveServo();
    }
  }
}//end loop
