#!/usr/bin/env python
# _*_ coding:utf-8 _*_

"""
  Copyright: 2016-2018 ROS小课堂 www.corvin.cn
  Author: corvin
  Description:
  History:
      20181226: initial this code.
"""

from __future__ import print_function

import roslib; roslib.load_manifest('arm_keyboard_control')
import rospy

from arduino_servo_connect.srv import *
from arduino_servo_connect.msg import MoveServo
import sys, select, termios, tty

msg = r"""
Reading from the keyboard and Publishing to MoveServo topic!
--------------------------------------------------------------
            Joint 2:[y  u]
           //            \\
          //              \\
         //               Joint 3:[i  o] = Joint 4:[k  l] = Joint 5:[, .]
  Joint 1:[h  j]
       ||
       ||
  Joint 0:[n  m]
---------------------------------------------------------------

CTRL-C to quit
"""

pub = rospy.Publisher('/arduino_servo_node/moveServoTopic', MoveServo, queue_size = 1)
Degrees = []

keyArray = {
    'n':(0, 1), 'm':(0, 0),
    'h':(1, 1), 'j':(1, 0),
    'y':(2, 1), 'u':(2, 0),
    'i':(3, 1), 'o':(3, 0),
    'k':(4, 1), 'l':(4, 0),
    ',':(5, 1), '.':(5, 0),
}

def getKey():
    tty.setraw(sys.stdin.fileno())
    select.select([sys.stdin], [], [], 0)
    key = sys.stdin.read(1)
    termios.tcsetattr(sys.stdin, termios.TCSADRAIN, settings)
    return key

def sendMoveMsg(servoId, targetPos, delay):
        moveMsg = MoveServo()
        moveMsg.servoId = servoId
        moveMsg.targetPos = targetPos
        moveMsg.delay = delay
        pub.publish(moveMsg)

def calTargetPose(servoId, turnFlag):
    targetPose = Degrees[servoId]
    if turnFlag == 0:
        if targetPose > 0:
            targetPose -= 1
    else:
        if targetPose < 160:
            targetPose += 1

    return targetPose

if __name__=="__main__":
    settings = termios.tcgetattr(sys.stdin)
    rospy.init_node('robotArm_keyboard_control_node')
    getServoPoseService = rospy.ServiceProxy("/arduino_servo_node/get_all_servos_pos", GetAllServoPos)
    moveDelay = rospy.get_param("~delay", 40)
    status = 0

    try:
        print(msg)

        while(1):
            key = getKey()

            if key in keyArray.keys():
                servoID = keyArray[key][0]
                turnFlag = keyArray[key][1]

                servoDegrees = getServoPoseService()
                Degrees = servoDegrees.positions
                targetPose = calTargetPose(servoID, turnFlag)

                if (status == 24):
                    print(msg)
                status = (status + 1) % 25
            else:
                if (key == '\x03'):
                    break
            rospy.loginfo("servoID:" + str(servoID) + ", targetPose:" + str(targetPose))
            sendMoveMsg(servoID, targetPose, moveDelay)
    except Exception as e:
        print(e)
    finally:
        termios.tcsetattr(sys.stdin, termios.TCSADRAIN, settings)
