#!/usr/bin/env python
# _*_ coding: utf-8 _*_

'''
  Copyright: 2016-2019 ROS小课堂 www.corvin.cn
  Author: corvin
  Description: 这个代码主要就是为了将从moveIt生成的控制舵机移动的命令给转换下区间,
    由于生成的控制命令序列有些包含负数,所以需要将其转换到0-160度范围内servo才能执行.
  History:
    20190110: Initial this file.
'''

import roslib
import rospy
from arduino_servo_connect.msg import MoveServo


class ARMJoint:
    def __init__(self, name, servoID):
        self.name = name
        self.servoID = servoID
        self.delay = 40
        self.targetPose = 0
        self.servoPub = rospy.Publisher("/arduino_servo_node/moveServoTopic", MoveServo, queue_size=5)


    def moveServoTarget(self):
        self.checkTargetPose()

        moveServoMsg = MoveServo()
        moveServoMsg.servoId = self.servoID
        moveServoMsg.targetPos = self.targetPose
        moveServoMsg.delay = self.delay
        self.servoPub.publish(moveServoMsg)
        #rospy.logwarn("ServoID:" + str(self.servoID) + ",TargetPose:" + str(self.targetPose))


    def checkTargetPose(self):
        if self.servoID == 0:
            self.targetPose += 80
        elif self.servoID == 1:
            self.targetPose = abs(self.targetPose - 80)
        elif self.servoID == 2:
            self.targetPose += 80
        elif self.servoID == 3:
            self.targetPose += 80
        elif self.servoID == 4:
            self.targetPose = abs(self.targetPose - 80)
        else:
            rospy.logerr("ServoID error:"+str(self.servoID))

