#!/usr/bin/env python
# _*_ coding:utf-8 _*_

import roslib
import rospy,os,sys
from math import radians
from arduino_servo_connect.srv import GetAllServoPos
from sensor_msgs.msg import JointState
from std_msgs.msg import Header

class PubJointState():
    def __init__(self):
        rospy.init_node('pub_joint_state_node', log_level=rospy.INFO)
        self.name = rospy.get_name()
        rospy.on_shutdown(self.shutdown)
        rospy.sleep(5)

        rospy.wait_for_service("/arduino_servo_node/get_all_servos_pos")
        try:
            self.getServoPoseSrv = rospy.ServiceProxy("/arduino_servo_node/get_all_servos_pos", GetAllServoPos)
        except rospy.ServiceException as e:
            rospy.logerr("Service call error:"+str(e))

        self.pub_joint = rospy.Publisher('/move_group/controller_joint_states', JointState, queue_size=10)
        self.rate = rospy.Rate(10)
        self.allServoDeg = ['','','','','']
        self.joint_msg = JointState()
        self.joint_msg.header = Header()
        self.joint_msg.name = ['servo0_joint','servo1_joint','servo2_joint','servo3_joint','servo4_joint']

        while not rospy.is_shutdown():
            servoDegSrv = self.getServoPoseSrv()

            for i in range(0, 5):
                self.allServoDeg[i] = radians(servoDegSrv.positions[i])
                #rospy.loginfo(str(self.allServoDeg[i]))

            self.procServoDeg()
            self.pubServoPose()
            self.rate.sleep()
            #rospy.loginfo("----------------------------")

    def procServoDeg(self):
        #servo 0
        self.allServoDeg[0] -= 1.396263;

        #servo 1
        if self.allServoDeg[1] >= 0 and self.allServoDeg[1] <= 1.396263:
            self.allServoDeg[1] = abs(self.allServoDeg[1] - 1.396263)
        else:
            self.allServoDeg[1] = -abs(self.allServoDeg[1] - 1.396263)

        #servo 2
        self.allServoDeg[2] -= 1.396263

        #servo 3
        self.allServoDeg[3] -= 1.396263

        #servo 4
        if self.allServoDeg[4] >= 0 and self.allServoDeg[4] <= 1.396263:
            self.allServoDeg[4] = abs(self.allServoDeg[4] - 1.396263)
        else:
            self.allServoDeg[4] = -abs(self.allServoDeg[4] - 1.396263)

    def pubServoPose(self):
        self.joint_msg.header.stamp = rospy.Time.now()
        self.joint_msg.position = self.allServoDeg
        self.pub_joint.publish(self.joint_msg)


    def shutdown(self):
        rospy.logwarn("shutdown pub_joint_status_node now ...")


if __name__ == '__main__':
    try:
        myJointState = PubJointState()
    except Exception as e:
        rospy.logerr("ErrorMsg: " + str(e))
        os._exit(1)
