#!/usr/bin/env python

import rospy, sys
import moveit_commander

class MoveItDemo:
    def __init__(self):
        #Initialize the move_group API
        moveit_commander.roscpp_initialize(sys.argv)

        #Initialize the ROS node
        rospy.init_node('moveit_fk_demo', anonymous=True)

        # connect to the robot_arm move group
        robot_arm = moveit_commander.MoveGroupCommander('robot_arm')

        #start the arm in the "resting"
        robot_arm.set_named_target("resting")

        #plan a trajectory to the goal configuration
        traj=robot_arm.plan()

        #Execute the planned trajectory
        robot_arm.execute(traj)
        rospy.loginfo("After move arm to resting pose !!!")
        rospy.sleep(2)

        #set target joint values for the arm
        joint_positions = [-0.08, 1.2, 0.02, 0.08, -1.2]

        #Set the arm's goal configuration to the joint position
        robot_arm.set_joint_value_target(joint_positions)

        #Plan and execute a trajectory to the goal configuration
        robot_arm.go()
        rospy.loginfo("After move arm to custom joint pose !!!")
        rospy.sleep(2)

        #Save this configuration for later
        robot_arm.remember_joint_values('saved_config', joint_positions)

        #Set the arm target to the "straight_up
        robot_arm.set_named_target("straight_up")
        robot_arm.go()
        rospy.loginfo("After move arm to straight_up pose !!!")
        rospy.sleep(2)

        #set the goal configuration to the named configuration saved
        robot_arm.set_named_target('saved_config')
        robot_arm.go()
        rospy.loginfo("After move arm to saved_config pose !!!")
        rospy.sleep(2)

        #Return the arm to default pose
        robot_arm.set_named_target("resting")
        robot_arm.go()
        rospy.loginfo("After move arm to resting pose !!!")
        rospy.sleep(2)

        #Cleanly shutdown moveIt!
        moveit_commander.roscpp_shutdown()

        # Exit the script
        moveit_commander.os._exit(0)

if __name__ == "__main__":
    try:
        MoveItDemo()
    except rospy.ROSInterruptException():
        pass
